@@ -28,7 +28,7 @@ __all__ = [
     "write_mem",
 ]
 
-__version__ = "4.5.1"
+__version__ = "4.6.2"
 
 import argparse
 import inspect
@@ -39,6 +39,7 @@ import time
 import traceback
 
 from esptool.cmds import (
+    DETECTED_FLASH_SIZES,
     chip_id,
     detect_chip,
     detect_flash_size,
@@ -354,7 +355,7 @@ def main(argv=None, esp=None):
     subparsers.add_parser("run", help="Run application code in flash")
 
     parser_image_info = subparsers.add_parser(
-        "image_info", help="Dump headers from an application image"
+        "image_info", help="Dump headers from a binary file (bootloader or application)"
     )
     parser_image_info.add_argument("filename", help="Image file to parse")
     parser_image_info.add_argument(
@@ -526,7 +527,9 @@ def main(argv=None, esp=None):
     add_spi_connection_arg(parser_read_flash)
     parser_read_flash.add_argument("address", help="Start address", type=arg_auto_int)
     parser_read_flash.add_argument(
-        "size", help="Size of region to dump", type=arg_auto_int
+        "size",
+        help="Size of region to dump. Use `ALL` to read to the end of flash.",
+        type=arg_auto_size,
     )
     parser_read_flash.add_argument("filename", help="Name of binary dump")
     parser_read_flash.add_argument(
@@ -570,8 +573,9 @@ def main(argv=None, esp=None):
     )
     parser_erase_region.add_argument(
         "size",
-        help="Size of region to erase (must be multiple of 4096)",
-        type=arg_auto_int,
+        help="Size of region to erase (must be multiple of 4096). "
+        "Use `ALL` to erase to the end of flash.",
+        type=arg_auto_size,
     )
 
     parser_merge_bin = subparsers.add_parser(
@@ -709,7 +713,16 @@ def main(argv=None, esp=None):
                 )
                 args.no_stub = True
             else:
-                esp = esp.run_stub()
+                try:
+                    esp = esp.run_stub()
+                except Exception:
+                    # The CH9102 bridge (PID: 0x55D4) can have issues on MacOS
+                    if sys.platform == "darwin" and esp._get_pid() == 0x55D4:
+                        print(
+                            "\nNote: If issues persist, "
+                            "try installing the WCH USB-to-Serial MacOS driver."
+                        )
+                    raise
 
         if args.override_vddsdio:
             esp.override_vddsdio(args.override_vddsdio)
@@ -811,16 +824,39 @@ def main(argv=None, esp=None):
 
         if hasattr(args, "flash_size"):
             print("Configuring flash size...")
-            detect_flash_size(esp, args)
-            if args.flash_size != "keep":  # TODO: should set this even with 'keep'
-                esp.flash_set_parameters(flash_size_bytes(args.flash_size))
+            if args.flash_size == "detect":
+                flash_size = detect_flash_size(esp, args)
+            elif args.flash_size == "keep":
+                flash_size = detect_flash_size(esp, args=None)
+            else:
+                flash_size = args.flash_size
+
+            if flash_size is not None:  # Secure download mode
+                esp.flash_set_parameters(flash_size_bytes(flash_size))
                 # Check if stub supports chosen flash size
-                if esp.IS_STUB and args.flash_size in ("32MB", "64MB", "128MB"):
+                if esp.IS_STUB and flash_size in ("32MB", "64MB", "128MB"):
                     print(
                         "WARNING: Flasher stub doesn't fully support flash size larger "
                         "than 16MB, in case of failure use --no-stub."
                     )
 
+        if getattr(args, "size", "") == "all":
+            if esp.secure_download_mode:
+                raise FatalError(
+                    "Detecting flash size is not supported in secure download mode. "
+                    "Set an exact size value."
+                )
+            # detect flash size
+            flash_id = esp.flash_id()
+            size_id = flash_id >> 16
+            size_str = DETECTED_FLASH_SIZES.get(size_id)
+            if size_str is None:
+                raise FatalError(
+                    "Detecting flash size failed. Set an exact size value."
+                )
+            print(f"Detected flash size: {size_str}")
+            args.size = flash_size_bytes(size_str)
+
         if esp.IS_STUB and hasattr(args, "address") and hasattr(args, "size"):
             if args.address + args.size > 0x1000000:
                 print(
@@ -865,6 +901,11 @@ def arg_auto_int(x):
     return int(x, 0)
 
 
+def arg_auto_size(x):
+    x = x.lower()
+    return x if x == "all" else arg_auto_int(x)
+
+
 def get_port_list():
     if list_ports is None:
         raise FatalError(
