@@ -4,6 +4,7 @@
 # SPDX-License-Identifier: GPL-2.0-or-later
 
 import struct
+import time
 
 from ..loader import ESPLoader
 from ..util import FatalError, NotSupportedError
@@ -58,6 +59,10 @@ class ESP32ROM(ESPLoader):
 
     XTAL_CLK_DIVIDER = 1
 
+    RTCCALICFG1 = 0x3FF5F06C
+    TIMERS_RTC_CALI_VALUE = 0x01FFFFFF
+    TIMERS_RTC_CALI_VALUE_S = 7
+
     FLASH_SIZES = {
         "1MB": 0x00,
         "2MB": 0x10,
@@ -148,13 +153,10 @@ class ESP32ROM(ESPLoader):
             return 0xF
 
     def get_encrypted_download_disabled(self):
-        if (
+        return (
             self.read_reg(self.EFUSE_DIS_DOWNLOAD_MANUAL_ENCRYPT_REG)
             & self.EFUSE_DIS_DOWNLOAD_MANUAL_ENCRYPT
-        ):
-            return True
-        else:
-            return False
+        )
 
     def get_flash_encryption_enabled(self):
         flash_crypt_cnt = (
@@ -281,8 +283,10 @@ class ESP32ROM(ESPLoader):
     def chip_id(self):
         raise NotSupportedError(self, "chip_id")
 
-    def read_mac(self):
+    def read_mac(self, mac_type="BASE_MAC"):
         """Read MAC from EFUSE region"""
+        if mac_type != "BASE_MAC":
+            return None
         words = [self.read_efuse(2), self.read_efuse(1)]
         bitstring = struct.pack(">II", *words)
         bitstring = bitstring[2:8]  # trim the 2 byte CRC
@@ -342,6 +346,33 @@ class ESP32ROM(ESPLoader):
                 progress_fn(len(data), length)
         return data
 
+    def get_rom_cal_crystal_freq(self):
+        """
+        Get the crystal frequency calculated by the ROM
+        """
+        # - Simulate the calculation in the ROM to get the XTAL frequency
+        #   calculated by the ROM
+
+        cali_val = (
+            self.read_reg(self.RTCCALICFG1) >> self.TIMERS_RTC_CALI_VALUE_S
+        ) & self.TIMERS_RTC_CALI_VALUE
+        clk_8M_freq = self.read_efuse(4) & (0xFF)  # EFUSE_RD_CK8M_FREQ
+        rom_calculated_freq = cali_val * 15625 * clk_8M_freq / 40
+        return rom_calculated_freq
+
+    def change_baud(self, baud):
+        # It's a workaround to avoid esp32 CK_8M frequency drift.
+        rom_calculated_freq = self.get_rom_cal_crystal_freq()
+        valid_freq = 40000000 if rom_calculated_freq > 33000000 else 26000000
+        false_rom_baud = int(baud * rom_calculated_freq // valid_freq)
+
+        print(f"Changing baud rate to {baud}")
+        self.command(self.ESP_CHANGE_BAUDRATE, struct.pack("<II", false_rom_baud, 0))
+        print("Changed.")
+        self._set_port_baudrate(baud)
+        time.sleep(0.05)  # get rid of garbage sent during baud rate change
+        self.flush_input()
+
 
 class ESP32StubLoader(ESP32ROM):
     """Access class for ESP32 stub loader, runs on top of ROM."""
@@ -354,7 +385,11 @@ class ESP32StubLoader(ESP32ROM):
         self.secure_download_mode = rom_loader.secure_download_mode
         self._port = rom_loader._port
         self._trace_enabled = rom_loader._trace_enabled
+        self.cache = rom_loader.cache
         self.flush_input()  # resets _slip_reader
 
+    def change_baud(self, baud):
+        ESPLoader.change_baud(self, baud)
+
 
 ESP32ROM.STUB_CLASS = ESP32StubLoader
