@@ -3,6 +3,8 @@
 #
 # SPDX-License-Identifier: GPL-2.0-or-later
 
+import os
+import re
 import struct
 import sys
 
@@ -70,6 +72,31 @@ def print_overwrite(message, last_line=False):
         print(message)
 
 
+def expand_chip_name(chip_name):
+    """Change chip name to official form, e.g. `esp32s3beta2` -> `ESP32-S3(beta2)`"""
+    # Put "-" after "esp32"
+    chip_name = re.sub(r"(esp32)(?!$)", r"\1-", chip_name)
+    # Put "()" around "betaN"
+    chip_name = re.sub(r"(beta\d*)", r"(\1)", chip_name)
+    # Uppercase everything before "(betaN)"
+    chip_name = re.sub(r"^[^\(]+", lambda x: x.group(0).upper(), chip_name)
+    return chip_name
+
+
+def strip_chip_name(chip_name):
+    """Strip chip name to normalized form, e.g. `ESP32-S3(beta2)` -> `esp32s3beta2`"""
+    return re.sub(r"[-()]", "", chip_name.lower())
+
+
+def get_file_size(path_to_file):
+    """Returns the file size in bytes"""
+    file_size = 0
+    with open(path_to_file, "rb") as f:
+        f.seek(0, os.SEEK_END)
+        file_size = f.tell()
+    return file_size
+
+
 class FatalError(RuntimeError):
     """
     Wrapper class for runtime errors that aren't caused by internal bugs, but by
@@ -87,6 +114,7 @@ class FatalError(RuntimeError):
         """
 
         err_defs = {
+            # ROM error codes
             0x101: "Out of memory",
             0x102: "Invalid argument",
             0x103: "Invalid state",
@@ -98,6 +126,18 @@ class FatalError(RuntimeError):
             0x109: "CRC or checksum was invalid",
             0x10A: "Version was invalid",
             0x10B: "MAC address was invalid",
+            # Flasher stub error codes
+            0xC000: "Bad data length",
+            0xC100: "Bad data checksum",
+            0xC200: "Bad blocksize",
+            0xC300: "Invalid command",
+            0xC400: "Failed SPI operation",
+            0xC500: "Failed SPI unlock",
+            0xC600: "Not in flash mode",
+            0xC700: "Inflate error",
+            0xC800: "Not enough data",
+            0xC900: "Too much data",
+            0xFF00: "Command not implemented",
         }
 
         err_code = struct.unpack(">H", result[:2])
